@@ -54,3 +54,8 @@
 
 #undef HAVE_STROPTS_H
 #undef HAVE_DLFCN_H
+
+@BOTTOM@
+#ifndef HAVE_HYPOT
+#define hypot(x, y) sqrt((x) * (x) + (y) * (y))
+#endif
