@@ -1,14 +1,14 @@
 #ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/debug.c,v 1.4 1998/12/03 06:29:09 sopwith Exp $";
+static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/debug.c,v 1.5 1999/02/25 01:05:05 unammx Exp $";
 #endif
 /*****
 * debug.c : debug initialization routines.
 *
-* This file Version	$Revision: 1.4 $
+* This file Version	$Revision: 1.5 $
 *
 * Creation date:		Fri Oct 18 23:57:23 GMT+0100 1996
-* Last modification: 	$Date: 1998/12/03 06:29:09 $
-* By:					$Author: sopwith $
+* Last modification: 	$Date: 1999/02/25 01:05:05 $
+* By:					$Author: unammx $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -35,6 +35,9 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/debug.
 /*****
 * ChangeLog 
 * $Log: debug.c,v $
+* Revision 1.5  1999/02/25 01:05:05  unammx
+* Missing bit of the strtok patches from Ulrich
+*
 * Revision 1.4  1998/12/03 06:29:09  sopwith
 *
 *
@@ -236,6 +239,7 @@ __rsd_selectDebugLevels(char *levels)
 	char *chPtr, *text;
 	int i;
 	int ret_val = 0;
+	char *tokp;
 
 	/* leave debuglevels alone if levels is not defined */
 	if(levels == NULL)
@@ -272,7 +276,7 @@ __rsd_selectDebugLevels(char *levels)
 		return(1);
 	}
 
-	for(chPtr = strtok(text, ","); chPtr != NULL; chPtr = strtok(NULL, ","))
+	for(chPtr = strtok_r(text, ",", &tokp); chPtr != NULL; chPtr = strtok_r(NULL, ",", &tokp))
 	{
 		i = 0;
 		i = atoi(chPtr);		
