@@ -1,13 +1,13 @@
 #ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/layout.c,v 1.4 1998/02/12 03:09:18 unammx Exp $";
+static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/layout.c,v 1.5 1999/06/02 01:00:42 unammx Exp $";
 #endif
 /*****
 * layout.c : XmHTML layout computation routines
 *
-* This file Version	$Revision: 1.4 $
+* This file Version	$Revision: 1.5 $
 *
 * Creation date:		Thu Nov  6 01:35:46 GMT+0100 1997
-* Last modification: 	$Date: 1998/02/12 03:09:18 $
+* Last modification: 	$Date: 1999/06/02 01:00:42 $
 * By:					$Author: unammx $
 * Current State:		$State: Exp $
 *
@@ -36,6 +36,33 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/layout
 /*****
 * ChangeLog 
 * $Log: layout.c,v $
+* Revision 1.5  1999/06/02 01:00:42  unammx
+*
+* 1999-06-01  Akira Higuchi <a-higuti@math.sci.hokudai.ac.jp>
+*
+* 	* libgnomeui/gnome-canvas-text.c:
+* 	* libgnomeui/gnome-icon-item.c:
+* 	* libgnomeui/gnome-less.c: Replace some gdk_font_load() calls with
+* 	gdk_fontset_load.    Use a more open fontset rule to load the fonts.
+*
+* 1999-06-01  Akira Higuchi <a-higuti@math.sci.hokudai.ac.jp>
+*
+* 	* gtk-xmhtml/XmHTMLP.h: Add three members lbearing, rbearing,
+* 	and width. These members are computed in allocFont().
+*
+* 	* gtk-xmhtml/toolkit.h: Remove Toolkit_XFont() macro.
+*
+* 	* gtk-xmhtml/XmHTML.c:
+* 	* gtk-xmhtml/fonts.c:
+* 	* gtk-xmhtml/format.c:
+* 	* gtk-xmhtml/gtk-xmhtml.c:
+* 	* gtk-xmhtml/layout.c:
+* 	* gtk-xmhtml/paint.c: Add fontset support. We use gdk_fontset_load()
+* 	instead of gdk_font_load() iff a fontset is supplied for label
+* 	widgets.
+*
+* 	* gtk-xmhtml/test.c: Add gtk_set_locale() call before gtk_init().
+*
 * Revision 1.4  1998/02/12 03:09:18  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
@@ -230,12 +257,6 @@ static int total_iterations;
 	} \
 }
 
-#ifdef WITH_MOTIF
-#   define XF(x) (x)->xfont
-#else
-#   define XF(x) ((XFontStruct *)((GdkFontPrivate *)x)->xfont)
-#endif
-
 /*****
 * Name:			_XmHTMLComputeLayout
 * Return Type:	void
@@ -1479,10 +1500,10 @@ ComputeTextLayout(XmHTMLWidget html, PositionBox *box, XmHTMLWord **words,
 	/* store final box height */
 	if(first_line || (box->height = box->y - y_start) == 0)
 	{
-		if(lineheight > FONTHEIGHT(XF (base_obj->font)))
+		if(lineheight > base_obj->font->height)
 			box->height = lineheight;
 		else
-			box->height = FONTHEIGHT(XF (base_obj->font));
+			box->height = base_obj->height;
 	}
 	else
 		box->height = max_box_height;
@@ -1495,8 +1516,8 @@ ComputeTextLayout(XmHTMLWidget html, PositionBox *box, XmHTMLWord **words,
 	box->width = max_box_width > min_box_width ? max_box_width : min_box_width;
 	box->min_width = min_box_width;
 
-	box->width += XF (base_obj->font)->max_bounds.lbearing;
-	box->min_width += XF (base_obj->font)->max_bounds.lbearing;
+	box->width += base_obj->font->lbearing;
+	box->min_width += base_obj->font->lbearing;
 
 	/* and check against document maximum width */
 	if(max_box_width > max_width)
