@@ -20,8 +20,8 @@
 #ifndef __ART_BPATH_H__
 #define __ART_BPATH_H__
 
-#include <art_point.h>
-#include <art_pathcode.h>
+#include <libart_lgpl/art_point.h>
+#include <libart_lgpl/art_pathcode.h>
 
 /* Basic data structures and constructors for bezier paths */
 
