@@ -654,7 +654,9 @@ add_digit(GtkWidget *w, gpointer data)
 	}
 
 	strcat(gc->result_string,digit);
-
+	if ((strcmp (gc->result_string, "0") != 0) || (strcmp (digit, "0") != 0))
+		strcat (gc->result_string, digit);
+	
 	put_led_font(gc);
 
 	sscanf(gc->result_string,"%lf",&gc->result);
@@ -770,12 +772,11 @@ c_fact(gdouble arg1)
 		errno=ERANGE;
 		return 0;
 	}
-	i = (int)arg1;
-        if(fabs(arg1-i)<1e-9) {
+	i = (int)(arg1+0.5);
+	if((fabs(arg1-i))>1e-9) {
 		errno=ERANGE;
 		return 0;
 	}
-        i = (int)(arg1+0.5);
 	for(r=1;i>0;i--)
 		r*=i;
 
