@@ -423,7 +423,7 @@ reconfigure_arrows (GnomeCanvasLine *line)
 
 		dx = poly[0] - line->coords[2];
 		dy = poly[1] - line->coords[3];
-		length = sqrt (dx * dx + dy * dy);
+		length = hypot(dx, dy);
 		if (length < GNOME_CANVAS_EPSILON)
 			sin_theta = cos_theta = 0.0;
 		else {
@@ -467,7 +467,7 @@ reconfigure_arrows (GnomeCanvasLine *line)
 
 		dx = poly[0] - coords[0];
 		dy = poly[1] - coords[1];
-		length = sqrt (dx * dx + dy * dy);
+		length = hypot(dx, dy);
 		if (length < GNOME_CANVAS_EPSILON)
 			sin_theta = cos_theta = 0.0;
 		else {
@@ -937,11 +937,11 @@ gnome_canvas_line_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_pa
 			width = 0.5;
 		
 		svp = art_svp_vpath_stroke (vpath,
-					    ART_PATH_STROKE_JOIN_MITER,
-					    ART_PATH_STROKE_CAP_BUTT,
+					    gnome_canvas_join_gdk_to_art (line->join),
+					    gnome_canvas_cap_gdk_to_art (line->cap),
 					    width,
 					    4,
-					    0.5);
+					    0.25);
 		art_free (vpath);
 
 		gnome_canvas_item_update_svp_clip (item, &line->fill_svp, svp, clip_path);
@@ -1164,7 +1164,7 @@ gnome_canvas_line_point (GnomeCanvasItem *item, double x, double y,
 		    || ((line->join == GDK_JOIN_ROUND) && (i != num_points))) {
 			dx = coords[0] - x;
 			dy = coords[1] - y;
-			dist = sqrt (dx * dx + dy * dy) - width / 2.0;
+			dist = hypot(dx, dy) - width / 2.0;
 			if (dist < GNOME_CANVAS_EPSILON) {
 				best = 0.0;
 				goto done;
@@ -1246,7 +1246,7 @@ gnome_canvas_line_point (GnomeCanvasItem *item, double x, double y,
 	if (line->cap == GDK_CAP_ROUND) {
 		dx = coords[0] - x;
 		dy = coords[1] - y;
-		dist = sqrt (dx * dx + dy * dy) - width / 2.0;
+		dist = hypot(dx, dy) - width / 2.0;
 		if (dist < GNOME_CANVAS_EPSILON) {
 			best = 0.0;
 			goto done;
