@@ -450,6 +450,15 @@ gnome_canvas_polygon_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		gnome_canvas_item_request_update (item);
 		break;
 
+	case ARG_OUTLINE_COLOR_RGBA:
+		poly->outline_set = TRUE;
+		poly->outline_rgba = GTK_VALUE_UINT (*arg);
+
+		/* should probably request repaint on the outline_svp */
+		gnome_canvas_item_request_update (item);
+
+		break;
+
 	case ARG_FILL_STIPPLE:
 		set_stipple (poly->fill_gc, &poly->fill_stipple, GTK_VALUE_BOXED (*arg), FALSE);
 		gnome_canvas_item_request_update (item);
@@ -532,6 +541,10 @@ gnome_canvas_polygon_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_UINT (*arg) = poly->fill_color;
 		break;
 
+	case ARG_OUTLINE_COLOR_RGBA:
+		GTK_VALUE_UINT (*arg) = poly->outline_color;
+		break;
+
 	case ARG_FILL_STIPPLE:
 		GTK_VALUE_BOXED (*arg) = poly->fill_stipple;
 		break;
@@ -610,8 +623,8 @@ gnome_canvas_polygon_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip
 				width = 0.5;
 		
 			svp = art_svp_vpath_stroke (vpath,
-						    ART_PATH_STROKE_JOIN_MITER,
-						    ART_PATH_STROKE_CAP_BUTT,
+						    ART_PATH_STROKE_JOIN_ROUND,
+						    ART_PATH_STROKE_CAP_ROUND,
 						    width,
 						    4,
 						    0.5);
