@@ -28,7 +28,6 @@
 #include <libart_lgpl/art_uta_svp.h>
 #include <libart_lgpl/art_rect_svp.h>
 
-
 /**
  * gnome_canvas_points_new:
  * @num_points: The number of points to allocate space for in the array.
@@ -195,7 +194,7 @@ gnome_canvas_get_butt_points (double x1, double y1, double x2, double y2,
 	width *= 0.5;
 	dx = x2 - x1;
 	dy = y2 - y1;
-	length = sqrt (dx * dx + dy * dy);
+	length = hypot(dx, dy);
 
 	if (length < GNOME_CANVAS_EPSILON) {
 		*bx1 = *bx2 = x2;
@@ -334,7 +333,7 @@ gnome_canvas_polygon_to_point (double *poly, int num_points, double x, double y)
 
 		dx = x - px;
 		dy = y - py;
-		dist = sqrt (dx * dx + dy * dy);
+		dist = hypot(dx, dy);
 		if (dist < best)
 			best = dist;
 	}
@@ -363,10 +362,36 @@ void
 gnome_canvas_render_svp (GnomeCanvasBuf *buf, ArtSVP *svp, guint32 rgba)
 {
 	guint32 fg_color, bg_color;
+	int alpha;
 
 	if (buf->is_bg) {
 		bg_color = buf->bg_color;
-		fg_color = rgba >> 8; /* FIXME: this needs to be a composite */
+		alpha = rgba & 0xff;
+		if (alpha == 0xff)
+			fg_color = rgba >> 8;
+		else {
+			/* composite over background color */
+			int bg_r, bg_g, bg_b;
+			int fg_r, fg_g, fg_b;
+			int tmp;
+
+			bg_r = (bg_color >> 16) & 0xff;
+			fg_r = (rgba >> 24) & 0xff;
+			tmp = (fg_r - bg_r) * alpha;
+			fg_r = bg_r + ((tmp + (tmp >> 8) + 0x80) >> 8);
+
+			bg_g = (bg_color >> 8) & 0xff;
+			fg_g = (rgba >> 16) & 0xff;
+			tmp = (fg_g - bg_g) * alpha;
+			fg_g = bg_g + ((tmp + (tmp >> 8) + 0x80) >> 8);
+
+			bg_b = bg_color & 0xff;
+			fg_b = (rgba >> 8) & 0xff;
+			tmp = (fg_b - bg_b) * alpha;
+			fg_b = bg_b + ((tmp + (tmp >> 8) + 0x80) >> 8);
+
+			fg_color = (fg_r << 16) | (fg_g << 8) | fg_b;
+		}
 		art_rgb_svp_aa (svp,
 				buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
 				fg_color, bg_color,
@@ -586,3 +611,58 @@ gnome_canvas_buf_ensure_buf (GnomeCanvasBuf *buf)
 		buf->is_buf = 1;
 	}
 }
+
+/**
+ * gnome_canvas_join_gdk_to_art
+ * @gdk_join: a join type, represented in GDK format
+ *
+ * Convert from GDK line join specifier to libart.
+ *
+ * Return value: The line join specifier in libart format.
+ **/
+ArtPathStrokeJoinType
+gnome_canvas_join_gdk_to_art (GdkJoinStyle gdk_join)
+{
+	switch (gdk_join) {
+	case GDK_JOIN_MITER:
+		return ART_PATH_STROKE_JOIN_MITER;
+
+	case GDK_JOIN_ROUND:
+		return ART_PATH_STROKE_JOIN_ROUND;
+
+	case GDK_JOIN_BEVEL:
+		return ART_PATH_STROKE_JOIN_BEVEL;
+
+	default:
+		g_assert_not_reached ();
+		return ART_PATH_STROKE_JOIN_MITER; /* shut up the compiler */
+	}
+}
+
+/**
+ * gnome_canvas_cap_gdk_to_art
+ * @gdk_cap: a cap type, represented in GDK format
+ *
+ * Convert from GDK line cap specifier to libart.
+ *
+ * Return value: The line cap specifier in libart format.
+ **/
+ArtPathStrokeCapType
+gnome_canvas_cap_gdk_to_art (GdkCapStyle gdk_cap)
+{
+	switch (gdk_cap) {
+	case GDK_CAP_BUTT:
+	case GDK_CAP_NOT_LAST:
+		return ART_PATH_STROKE_CAP_BUTT;
+
+	case GDK_CAP_ROUND:
+		return ART_PATH_STROKE_CAP_ROUND;
+
+	case GDK_CAP_PROJECTING:
+		return ART_PATH_STROKE_CAP_SQUARE;
+
+	default:
+		g_assert_not_reached ();
+		return ART_PATH_STROKE_CAP_BUTT; /* shut up the compiler */
+	}
+}
