@@ -314,7 +314,9 @@ void gnome_less_set_fixed_font  (GnomeLess * gl, gboolean fixed)
     /* Is this a standard X font in every respect? */
     /* I'm told that "fixed" is the standard X font, but
        "fixed" doesn't appear to be fixed-width; courier does though. */
-    font = gdk_font_load("-*-courier-medium-r-normal-*-12-*-*-*-*-*-*-*");
+    font = gdk_fontset_load(
+	"-*-courier-medium-r-normal-*-12-*-*-*-*-*-*-*"
+	"-*-*-medium-r-normal-*-12-*-*-*-*-*-*-*,*");
     
     if ( font == NULL ) {
       g_warning("GnomeLess: Couldn't load fixed font\n");
