@@ -177,6 +177,15 @@ enum_from_strings(gint * setme, gchar * findme,
   return retval;
 }
 
+/**
+ * gnome_preferences_load_custom
+ * @settings: App-specified set of user preferences
+ *
+ * Description:
+ * Uses gnome_config_xxx() interface to load a set of
+ * standard GNOME preferences into the specified @settings object.
+ **/
+ 
 void
 gnome_preferences_load_custom(GnomePreferences *settings)
 {
@@ -326,6 +335,15 @@ gnome_preferences_load_custom(GnomePreferences *settings)
   gnome_config_pop_prefix();
 }
 
+/**
+ * gnome_preferences_save_custom
+ * @settings: App-specified set of user preferences
+ *
+ * Description:
+ * Uses gnome_config_xxx() interface to store a set of
+ * standard GNOME preferences from info in the @settings object.
+ **/
+ 
 void
 gnome_preferences_save_custom(GnomePreferences *settings)
 {
@@ -389,144 +407,384 @@ gnome_preferences_save_custom(GnomePreferences *settings)
   gnome_config_sync();
 }
 
+/**
+ * gnome_preferences_load
+ *
+ * Description:
+ * Uses gnome_config_xxx() API to load a standard set of GNOME
+ * preferences into the default GNOME preferences object.
+ **/
+ 
 void gnome_preferences_load(void)
 {
   gnome_preferences_load_custom(&prefs);
 }
 
+/**
+ * gnome_preferences_save
+ *
+ * Description:
+ * Uses gnome_config_xxx() API to store a standard set of GNOME
+ * preferences using info in the default GNOME preferences object.
+ **/
+ 
 void gnome_preferences_save(void)
 {
   gnome_preferences_save_custom(&prefs);
 }
 
+
+/**
+ * gnome_preferences_get_button_layout 
+ *
+ * Description:
+ * Obtain the button style from the default GNOME preferences object.
+ *
+ * Returns:
+ * Enumerated type indicating the default GNOME dialog button style.
+ **/
+    
 GtkButtonBoxStyle gnome_preferences_get_button_layout (void)
 {
   return prefs.dialog_buttons_style;
 }
 
+
+/**
+ * gnome_preferences_set_button_layout 
+ * style: Enumerated type indicating the default GNOME dialog button style.
+ *
+ * Description:
+ * Set the default GNOME preferences object's default button style.
+ **/
+
 void              gnome_preferences_set_button_layout (GtkButtonBoxStyle style)
 {
   g_return_if_fail ((style <= GTK_BUTTONBOX_END));
   prefs.dialog_buttons_style = style;
 }
 
+
+/**
+ * gnome_preferences_get_statusbar_dialog     
+ *
+ * Description:
+ * Determine whether or not the statusbar is a dialog.
+ *
+ * Returns:
+ * %FALSE if statusbar is a dialog, %TRUE if not.
+ **/
+
 gboolean          gnome_preferences_get_statusbar_dialog     (void)
 {
   return prefs.statusbar_not_dialog;
 }
 
+
+/**
+ * gnome_preferences_set_statusbar_dialog     
+ * b: %FALSE if statusbar is a dialog, %TRUE if not.
+ *
+ * Description:
+ * Indicate whether or not the default for GNOME status bars
+ * is a dialog.
+ **/
+
 void              gnome_preferences_set_statusbar_dialog     (gboolean b)
 {
   prefs.statusbar_not_dialog = b;
 }
 
+
+/**
+ * gnome_preferences_get_statusbar_interactive
+ *
+ * Description:
+ * Determine whether or not the statusbar is interactive.
+ *
+ * Returns:
+ * %TRUE if statusbar is interactive, %FALSE if not.
+ **/
+
 gboolean          gnome_preferences_get_statusbar_interactive(void)
 {
   return prefs.statusbar_is_interactive;
 }
 
+
+/**
+ * gnome_preferences_set_statusbar_interactive
+ * b: %TRUE if statusbar is interactive, %FALSE if not.
+ *
+ * Description:
+ * Indicate whether or not the GNOME status bars are, by default,
+ * interactive.
+ **/
+
 void              gnome_preferences_set_statusbar_interactive(gboolean b)
 {
   prefs.statusbar_is_interactive = b;
 }
 
-gboolean
-gnome_preferences_get_statusbar_meter_on_right (void)
+
+/**
+ * gnome_preferences_get_statusbar_meter_on_right 
+ *
+ * Description:
+ * Determine whether or not the statusbar's meter is on the right-hand side. 
+ *
+ * Returns:
+ * %TRUE if statusbar meter is on the right side, %FALSE if not.
+ **/
+
+gboolean gnome_preferences_get_statusbar_meter_on_right (void)
 {
   return prefs.statusbar_meter_on_right;
 }
 
+
+/**
+ * gnome_preferences_set_statusbar_meter_on_right 
+ * statusbar_meter_on_right: %TRUE if statusbar meter is on the right side, %FALSE if not.
+ *
+ * Description:
+ * Indicate whether or not the GNOME status bars are, by default,
+ * on the right-hand side.
+ **/
+
 void
 gnome_preferences_set_statusbar_meter_on_right (gboolean statusbar_meter_on_right)
 {
   prefs.statusbar_meter_on_right = statusbar_meter_on_right;
 }
 
+
+/**
+ * gnome_preferences_get_menubar_detachable   
+ *
+ * Description:
+ * Determine whether or not a menu bar is, by default,
+ * detachable from its parent frame.
+ *
+ * Returns:
+ * %TRUE if menu bars are detachable, %FALSE if not.
+ **/
+
 gboolean          gnome_preferences_get_menubar_detachable   (void)
 {
   return prefs.menubar_detachable;
 }
 
+
+/**
+ * gnome_preferences_set_menubar_detachable   
+ * b: %TRUE if menu bars are detachable, %FALSE if not.
+ *
+ * Description:
+ * Indicate whether or not the GNOME menu bars are, by default,
+ * detachable from their parent frame.
+ **/
+
 void              gnome_preferences_set_menubar_detachable   (gboolean b)
 {
   prefs.menubar_detachable = b;
 }
 
+
+/**
+ * gnome_preferences_get_menubar_relief    
+ *
+ * Description:
+ **/
+
 gboolean          gnome_preferences_get_menubar_relief    (void)
 {
   return prefs.menubar_relief;
 }
 
+
+/**
+ * gnome_preferences_set_menubar_relief    
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_menubar_relief    (gboolean b)
 {
   prefs.menubar_relief = b;
 }
 
+
+/**
+ * gnome_preferences_get_toolbar_detachable   
+ *
+ * Description:
+ **/
+
 gboolean          gnome_preferences_get_toolbar_detachable   (void)
 {
   return prefs.toolbar_detachable;
 }
 
+
+/**
+ * gnome_preferences_set_toolbar_detachable   
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_toolbar_detachable   (gboolean b)
 {
   prefs.toolbar_detachable = b;
 }
 
+
+/**
+ * gnome_preferences_get_toolbar_relief    
+ *
+ * Description:
+ **/
+
 gboolean          gnome_preferences_get_toolbar_relief    (void)
 {
   return prefs.toolbar_relief;
 }
 
+
+/**
+ * gnome_preferences_set_toolbar_relief    
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_toolbar_relief    (gboolean b)
 {
   prefs.toolbar_relief = b;
 }
 
+
+/**
+ * gnome_preferences_get_toolbar_relief_btn 
+ *
+ * Description:
+ **/
+
 gboolean          gnome_preferences_get_toolbar_relief_btn (void)
 {
   return prefs.toolbar_relief_btn;
 }
 
+
+/**
+ * gnome_preferences_set_toolbar_relief_btn 
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_toolbar_relief_btn (gboolean b)
 {
   prefs.toolbar_relief_btn = b;
 }
 
+
+/**
+ * gnome_preferences_get_toolbar_lines     
+ *
+ * Description:
+ **/
+
 gboolean          gnome_preferences_get_toolbar_lines     (void)
 {
   return prefs.toolbar_lines;
 }
 
+
+/**
+ * gnome_preferences_set_toolbar_lines     
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_toolbar_lines     (gboolean b)
 {
   prefs.toolbar_lines = b;
 }
 
+
+/**
+ * gnome_preferences_get_toolbar_labels    
+ *
+ * Description:
+ **/
+
 gboolean          gnome_preferences_get_toolbar_labels    (void)
 {
   return prefs.toolbar_labels;
 }
 
+
+/**
+ * gnome_preferences_set_toolbar_labels    
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_toolbar_labels    (gboolean b)
 {
   prefs.toolbar_labels = b;
 }
 
-gboolean          gnome_preferences_get_dialog_centered      ()
+
+/**
+ * gnome_preferences_get_dialog_centered      
+ *
+ * Description:
+ **/
+
+gboolean          gnome_preferences_get_dialog_centered      (void)
 {
   return prefs.dialog_centered;
 }
 
+
+/**
+ * gnome_preferences_set_dialog_centered      
+ * @b:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_dialog_centered      (gboolean b)
 {
   prefs.dialog_centered = b;
 }
 
-GtkWindowType     gnome_preferences_get_dialog_type          ()
+
+/**
+ * gnome_preferences_get_dialog_type          
+ *
+ * Description:
+ **/
+
+GtkWindowType     gnome_preferences_get_dialog_type          (void)
 {
   return prefs.dialog_type;
 }
 
+
+/**
+ * gnome_preferences_set_dialog_type          
+ * @t:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_dialog_type          (GtkWindowType t)
 {
   prefs.dialog_type = t;
@@ -534,80 +792,177 @@ void              gnome_preferences_set_dialog_type          (GtkWindowType t)
 
 /* Whether dialogs are GTK_WIN_POS_NONE, GTK_WIN_POS_CENTER,
    GTK_WIN_POS_MOUSE */
-GtkWindowPosition gnome_preferences_get_dialog_position      ()
+
+/**
+ * gnome_preferences_get_dialog_position      
+ *
+ * Description:
+ **/
+
+GtkWindowPosition gnome_preferences_get_dialog_position      (void)
 {
   return prefs.dialog_position;
 }
 
+
+/**
+ * gnome_preferences_set_dialog_position      
+ * @p:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_dialog_position      (GtkWindowPosition p)
 {
   prefs.dialog_position = p;
 }
 
-GnomeMDIMode      gnome_preferences_get_mdi_mode             ()
+
+/**
+ * gnome_preferences_get_mdi_mode             
+ *
+ * Description:
+ **/
+
+GnomeMDIMode      gnome_preferences_get_mdi_mode             (void)
 {
   return prefs.mdi_mode;
 }
 
+
+/**
+ * gnome_preferences_set_mdi_mode             
+ * @m:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_mdi_mode             (GnomeMDIMode m)
 {
   prefs.mdi_mode = m;
 }
 
-GtkPositionType   gnome_preferences_get_mdi_tab_pos          ()
+
+/**
+ * gnome_preferences_get_mdi_tab_pos          
+ *
+ * Description:
+ **/
+
+GtkPositionType   gnome_preferences_get_mdi_tab_pos          (void)
 {
   return prefs.mdi_tab_pos;
 }
 
+
+/**
+ * gnome_preferences_set_mdi_tab_pos          
+ * @p:
+ *
+ * Description:
+ **/
+
 void              gnome_preferences_set_mdi_tab_pos          (GtkPositionType p)
 {
   prefs.mdi_tab_pos = p;
 }
 
-int
-gnome_preferences_get_property_box_apply (void)
+
+/**
+ * gnome_preferences_get_property_box_apply 
+ *
+ * Description:
+ **/
+
+int gnome_preferences_get_property_box_apply (void)
 {
 	return prefs.property_box_buttons_apply;
 }
 
-void
-gnome_preferences_set_property_box_button_apply (int v)
+
+/**
+ * gnome_preferences_set_property_box_button_apply 
+ * @v:
+ *
+ * Description:
+ **/
+
+void gnome_preferences_set_property_box_button_apply (int v)
 {
 	prefs.property_box_buttons_apply = v;
 }
 
-gboolean
-gnome_preferences_get_menus_have_tearoff (void)
+
+/**
+ * gnome_preferences_get_menus_have_tearoff 
+ *
+ * Description:
+ **/
+
+gboolean gnome_preferences_get_menus_have_tearoff (void)
 {
 	return prefs.menus_have_tearoff;
 }
 
-void
-gnome_preferences_set_menus_have_tearoff (gboolean b)
+
+/**
+ * gnome_preferences_set_menus_have_tearoff 
+ * @b:
+ *
+ * Description:
+ **/
+
+void gnome_preferences_set_menus_have_tearoff (gboolean b)
 {
 	prefs.menus_have_tearoff = b;
 }
 
-int
-gnome_preferences_get_menus_have_icons (void)
+
+/**
+ * gnome_preferences_get_menus_have_icons 
+ *
+ * Description:
+ **/
+
+int gnome_preferences_get_menus_have_icons (void)
 {
 	return prefs.menus_have_icons;
 }
 
-void
-gnome_preferences_set_menus_have_icons (int have_icons)
+
+/**
+ * gnome_preferences_set_menus_have_icons 
+ * @have_icons:
+ *
+ * Description:
+ **/
+
+void gnome_preferences_set_menus_have_icons (int have_icons)
 {
 	prefs.menus_have_icons = have_icons ? TRUE : FALSE;
 }
 
-int
-gnome_preferences_get_disable_imlib_cache (void)
+
+/**
+ * gnome_preferences_get_disable_imlib_cache 
+ *
+ * Description:
+ **/
+
+int gnome_preferences_get_disable_imlib_cache (void)
 {
 	return prefs.disable_imlib_cache;
 }
 
-void
-gnome_preferences_set_disable_imlib_cache (int disable_imlib_cache)
+
+/**
+ * gnome_preferences_set_disable_imlib_cache 
+ * @disable_imlib_cache:
+ *
+ * Description:
+ **/
+
+void gnome_preferences_set_disable_imlib_cache (int disable_imlib_cache)
 {
 	prefs.disable_imlib_cache = disable_imlib_cache ? TRUE : FALSE;
 }
