@@ -41,7 +41,6 @@
 #include "vt.h"
 #include "subshell.h"
 
-
 /* define to 'x' to enable copius debug of this module */
 #define d(x)
 
@@ -326,7 +325,7 @@ vt_scroll_down(struct vt_em *vt, int count)
   struct vt_line *wn, *nn;
   int i;
   uint32 blank = vt->attr & VTATTR_CLEARMASK;
-  
+
   d(printf("vt_scroll_down count=%d top=%d bottom=%d\n",
 	   count, vt->scrolltop, vt->scrollbottom));
 
@@ -425,7 +424,7 @@ vt_erase_chars(struct vt_em *vt, int count)
   int i;
 
   l = vt->this_line;
-  for (i = vt->cursorx+count;i<l->width;i++)
+  for (i = vt->cursorx; i<vt->cursorx+count && i<l->width;i++)
     l->data[i] = vt->attr & VTATTR_CLEARMASK;
 }
 
@@ -637,7 +636,7 @@ static void vt_tab(struct vt_em *vt)
     /* We do not store the attribute as tabs are transparent
      * with respect to attributes
      */
-    l->data[vt->cursorx] = 9 | VTATTR_CLEAR;
+    l->data[vt->cursorx] = 9 | vt->attr;
   }
 
   /* move cursor to new tab position */
@@ -1022,10 +1021,19 @@ vt_goto(struct vt_em *vt)
 }
 
 /* goto a specific y location */
+/* also handles \Ed reset coding method */
 static void
 vt_gotoabsy(struct vt_em *vt)
 {
-  vt_gotoxy(vt, vt->cursorx, vt->arg.num.intargs[0]?vt->arg.num.intargs[0]-1:0);
+#ifdef ZVT_UTF
+  if (vt->state==1) {
+    vt->coding = ZVT_CODE_ISOLATIN1;
+  } else {
+#endif
+    vt_gotoxy(vt, vt->cursorx, vt->arg.num.intargs[0]?vt->arg.num.intargs[0]-1:0);
+#ifdef ZVT_UTF
+  }
+#endif
 }
 
 static void
@@ -1034,6 +1042,23 @@ vt_gotoabsx(struct vt_em *vt)
   vt_gotoxy(vt, vt->arg.num.intargs[0]?vt->arg.num.intargs[0]-1:0, vt->cursory);
 }
 
+/* set character encoding
+   \E%n sequence */
+static void
+vt_char_encoding(struct vt_em *vt)
+{
+#ifdef ZVT_UTF
+  switch (vt->arg.num.intargs[1]) {
+  case 'G':
+    vt->coding = ZVT_CODE_UTF8;
+    vt->decode.utf8.shiftchar = 0;
+    break;
+  default:
+    vt->coding = ZVT_CODE_ISOLATIN1;
+  }
+#endif
+}
+
 /* various mode stuff */
 static void
 vt_setmode(struct vt_em *vt, int on)
@@ -1365,7 +1390,7 @@ struct vt_jump vtjumps[] = {
   {0,0}, {0,0}, {0,0}, {0,0},	/* 18: ^X ^[ */
   {0,0}, {0,0}, {0,0}, {0,0},	/* 1c: ^\ ^] ^^ ^_  */
   {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT}, {vt_deccharmode,VT_LIT|VT_EXA},	/*  !"# */
-  {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT},	/* $%&' */
+  {0,VT_LIT}, {vt_char_encoding,VT_LIT|VT_EXA}, {0,VT_LIT}, {0,VT_LIT},	/* $%&' */
   {vt_gx_set,VT_LIT|VT_EXA}, {vt_gx_set,VT_LIT|VT_EXA}, {vt_gx_set,VT_LIT|VT_EXA}, {vt_gx_set,VT_LIT|VT_EXA},	/* ()*+ */
   {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT},	/* ,-./ */
   {0,VT_LIT|VT_ARG}, {0,VT_LIT|VT_ARG}, {0,VT_LIT|VT_ARG}, {0,VT_LIT|VT_ARG},	/* 0123 */
@@ -1381,7 +1406,7 @@ struct vt_jump vtjumps[] = {
   {vt_erase_char,VT_EBL}, {0,VT_LIT}, {vt_backtab,VT_EBL}, {0,VT_LIT},	/* XYZ[ */
   {0,VT_LIT}, {0,VT_LIT}, {vt_scroll_reverse,VT_EBL}, {0,VT_LIT},	/* \]^_ */
   {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT}, {vt_reset,VT_EXL|VT_EXB},	/* `abc */
-  {vt_gotoabsy,VT_EBL}, {0,VT_LIT}, {vt_goto,VT_EBL}, {0,VT_LIT},	/* defg */
+  {vt_gotoabsy,VT_EBL|VT_ESC}, {0,VT_LIT}, {vt_goto,VT_EBL}, {0,VT_LIT},	/* defg */
   {vt_modeh,VT_EBL}, {0,VT_LIT}, {0,VT_LIT}, {vt_modek,VT_EBL},	/* hijk */
   {vt_model,VT_EBL}, {vt_mode,VT_EBL}, {vt_dsr,VT_EBL}, {0,VT_LIT},	/* lmno */
   {vt_reset,VT_EBL}, {0,VT_LIT}, {vt_scroll,VT_EBL}, {0,VT_LIT},	/* pqrs */
@@ -1431,148 +1456,188 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 
     /* convert to unsigned byte */
     c = (*ptr++) & 0xff;
+
+#ifdef ZVT_UTF
+
+    if (vt->coding == ZVT_CODE_UTF8 && (c&0x80)!=0) {
+      if (c<0xc0) {
+	/* might be part of a multi-byte content ... */
+	if ((vt->decode.utf8.shiftchar & 0x80) == 0) {
+	  /* treat this character as a control character/international character.
+	     This (0x80-0xc0] should cover the C1 control set and some of the
+	     international characters in normal latin 1, but not all :( */
+	  mode = modes[c & 0x7f].modes;
+	  process = modes[c & 0x7f].process;
+	} else {
+	  vt->decode.utf8.shiftchar <<= 1;
+	  vt->decode.utf8.wchar = (vt->decode.utf8.wchar<<6) | (c&0x3f);
+	  if ((vt->decode.utf8.shiftchar & 0x80) == 0) {
+	    c = vt->decode.utf8.wchar|((vt->decode.utf8.shiftchar&0x7f)<<vt->decode.utf8.shift);
+	    /* and run with it! */
+	    vt->decode.utf8.shiftchar=0;
+
+	    /* all extended characters are just literals */
+	    mode = VT_LIT;
+	    process = 0;
+	  } else {
+	    vt->decode.utf8.shift+=5;
+	    continue;
+	  }
+	}
+      } else {
+	vt->decode.utf8.shiftchar=c<<1;
+	vt->decode.utf8.wchar=0;
+	vt->decode.utf8.shift=4;
+	continue;
+      }
+    } else {
+      mode = modes[c & 0x7f].modes;
+      process = modes[c & 0x7f].process;
+    }
+#else
     mode = modes[c & 0x7f].modes;
     process = modes[c & 0x7f].process;
+#endif /* ZVT_UTF */
 
     vt->state = state;		/* so callbacks know their state */
-
+    
     d(printf("state %d: %d $%02x '%c'\n",state, vt->argcnt, c, isprint(c)?c:'.'));
-
+    
     switch (state) {
-
-      case 0:
-	if (mode & VT_LIT) {
-	  /* remap character? */
-	  if (vt->remaptable)
-	    c=vt->remaptable[c];
-
-	  /* insert mode? */
-	  if (vt->mode & VTMODE_INSERT)
-	    vt_insert_chars(vt, 1);
-
-	  /* need to wrap? */
-	  if (vt->cursorx>=vt->width) {
-	    if (vt->mode&VTMODE_WRAPOFF)
-	      vt->cursorx = vt->width-1;
-	    else {
-	      vt_lf(vt);
-	      vt->cursorx=0;
-	    }
+      
+    case 0:
+      if (mode & VT_LIT) {
+	/* remap character? */
+	if (vt->remaptable && c<=0xff)
+	  c=vt->remaptable[c];
+	
+	/* insert mode? */
+	if (vt->mode & VTMODE_INSERT)
+	  vt_insert_chars(vt, 1);
+	
+	/* need to wrap? */
+	if (vt->cursorx>=vt->width) {
+	  if (vt->mode&VTMODE_WRAPOFF)
+	    vt->cursorx = vt->width-1;
+	  else {
+	    vt_lf(vt);
+	    vt->cursorx=0;
 	  }
-
-	  /* output character */
-	  vt->this_line->data[vt->cursorx] = ((vt->attr) & VTATTR_MASK) | c;
-	  vt->this_line->modcount++;
-	  /* d(printf("literal %c\n", c)); */
-	  vt->cursorx++;
-	} else if (mode & VT_CON) {
-	  process(vt);
-	} else if (c==27) {
-	  state=1;
 	}
-	/* else ignore */
-	break;
-
-
-	/* received 'esc', next byte */
-      case 1:
-	if (mode & VT_ESC) {	/* got a \Ex sequence */
-	  vt->argcnt = 0;
-	  vt->arg.num.intargs[0] = 0;
+	
+	/* output character */
+	vt->this_line->data[vt->cursorx] = ((vt->attr) & VTATTR_MASK) | c;
+	vt->this_line->modcount++;
+	/* d(printf("literal %c\n", c)); */
+	vt->cursorx++;
+      } else if (mode & VT_CON) {
+	process(vt);
+      } else if (c==27) {
+	state=1;
+      }
+      /* else ignore */
+      break;
+      
+      
+      /* received 'esc', next byte */
+    case 1:
+      if (mode & VT_ESC) {	/* got a \Ex sequence */
+	vt->argcnt = 0;
+	vt->arg.num.intargs[0] = 0;
+	process(vt);
+	state=0;
+      } else if (c=='[') {
+	vt->arg.num.intarg = 0;
+	vt->argcnt = 0;
+	state = 2;
+      } else if (c=='O') {
+	state = 3;
+      } else if (c==']') {	/* set text parameters, read parameters */
+	state = 4;
+	vt->arg.txt.outptr = vt->arg.txt.args_mem;
+      } else if (mode & VT_EXA) {
+	vt->arg.num.intargs[0] = c & 0x7f;
+	state = 5;
+      } else {
+	state = 0;		/* dont understand input */
+      }
+      break;
+      
+      
+    case 2:
+      if (c=='?') {
+	state = 6;
+	continue;
+      } else if (c=='!') {
+	state = 8;
+	continue;
+      }
+      /* note -> falls through */
+    case 6:
+    case 7:
+    case 8:
+    case 9:
+      if (mode & VT_ARG) {
+	/* accumulate subtotal */
+	vt->arg.num.intarg = vt->arg.num.intarg*10+(c-'0');
+      } else if ((mode & VT_EXB) || c==';' || c==':') {	/* looking for 'arg;arg;...' */
+	if (vt->argcnt < VTPARAM_INTARGS) {
+	  vt->arg.num.intargs[vt->argcnt] = vt->arg.num.intarg & 0x7fffffff;
+	  vt->argcnt++;
+	  vt->arg.num.intarg = 0;
+	}
+	if (mode & VT_EXB) {
 	  process(vt);
 	  state=0;
-	} else if (c=='[') {
-	  vt->arg.num.intarg = 0;
-	  vt->argcnt = 0;
-	  state = 2;
-	} else if (c=='O') {
-	  state = 3;
-	} else if (c==']') {	/* set text parameters, read parameters */
-	  state = 4;
-	  vt->arg.txt.outptr = vt->arg.txt.args_mem;
-	} else if (mode & VT_EXA) {
-	  vt->arg.num.intargs[0] = c & 0x7f;
-	  state = 5;
-	} else {
-	  state = 0;		/* dont understand input */
 	}
-	break;
-
-
-      case 2:
-	if (c=='?') {
-	  state = 6;
-	  continue;
-	} else if (c=='!') {
-	  state = 8;
-	  continue;
-	}
-	/* note -> falls through */
-      case 6:
-      case 7:
-      case 8:
-      case 9:
-	if (mode & VT_ARG) {
-	  /* accumulate subtotal */
-	  vt->arg.num.intarg = vt->arg.num.intarg*10+(c-'0');
-	} else if ((mode & VT_EXB) || c==';' || c==':') {	/* looking for 'arg;arg;...' */
-	  if (vt->argcnt < VTPARAM_INTARGS) {
-	    vt->arg.num.intargs[vt->argcnt] = vt->arg.num.intarg & 0x7fffffff;
-	    vt->argcnt++;
-	    vt->arg.num.intarg = 0;
-	  }
-	  if (mode & VT_EXB) {
-	    process(vt);
-	    state=0;
-	  }
-	} else if (mode & VT_CON) {
-	  process(vt);
-	} else if (c==' ') {	/* this expects the next char == VT_EXB.  close enough! */
-	  state = 7;		/* any other chars should be numbers anyway */
-	} else if (c=='\'') {
-	  state = 9;
-	} else {
-	  d(printf("unknown option '%c'\n", c));
-	  state=0;		/* unexpected escape sequence - ignore */
-	}
-	break;
-
-
-	/* \EOx */
-      case 3:
-	if (mode & VT_EXO) {
-	  vt->arg.num.intargs[0] = 0;
-	  process(vt);
-	}	/* ignore otherwise */
-	state=0;
-	break;
-
-
-	/* \E]..;...BEL */
-      case 4:
-	if (c==0x07) {
-	  /* handle output */
-	  *(vt->arg.txt.outptr)=0;
-	  vt_set_text(vt);
-	  d(printf("received text mode: %s\n", vt->arg.txt.args_mem));
-	  state = 0;
-	} else if (c==0x0a) {
-	  state = 0;		/* abort command */
-	} else {
-	  if (vt->arg.txt.outptr<(vt->arg.txt.args_mem+VTPARAM_MAXARGS*VTPARAM_ARGMAX-2)) /* truncate excessive args */
-	    *(vt->arg.txt.outptr)++=c;
-	}
-	break;
-
-
-	/* \E?x */
-      case 5:
-	vt->arg.num.intargs[1]=c;
-	vt->argcnt=0;
-	modes[vt->arg.num.intargs[0]].process(vt);
-	state=0;
-	break;
+      } else if (mode & VT_CON) {
+	process(vt);
+      } else if (c==' ') {	/* this expects the next char == VT_EXB.  close enough! */
+	state = 7;		/* any other chars should be numbers anyway */
+      } else if (c=='\'') {
+	state = 9;
+      } else {
+	d(printf("unknown option '%c'\n", c));
+	state=0;		/* unexpected escape sequence - ignore */
       }
+      break;
+      
+      
+      /* \EOx */
+    case 3:
+      if (mode & VT_EXO) {
+	vt->arg.num.intargs[0] = 0;
+	process(vt);
+      }	/* ignore otherwise */
+      state=0;
+      break;
+      
+      
+      /* \E]..;...BEL */
+    case 4:
+      if (c==0x07) {
+	/* handle output */
+	*(vt->arg.txt.outptr)=0;
+	vt_set_text(vt);
+	d(printf("received text mode: %s\n", vt->arg.txt.args_mem));
+	state = 0;
+      } else if (c==0x0a) {
+	state = 0;		/* abort command */
+      } else {
+	if (vt->arg.txt.outptr<(vt->arg.txt.args_mem+VTPARAM_MAXARGS*VTPARAM_ARGMAX-2)) /* truncate excessive args */
+	  *(vt->arg.txt.outptr)++=c;
+      }
+      break;
+      
+      
+      /* \E?x */
+    case 5:
+      vt->arg.num.intargs[1]=c;
+      vt->argcnt=0;
+      modes[vt->arg.num.intargs[0]].process(vt);
+      state=0;
+      break;
+    }
   }
 
   vt->state = state;
@@ -1675,6 +1740,12 @@ vt_init(struct vt_em *vt, int width, int height)
   vt->ring_my_bell = 0L;
   vt->change_my_name = 0L;
 
+#ifdef ZVT_UTF
+  vt->decode.utf8.shiftchar = 0;
+  /* we actually start in isolatin1 mode */
+  vt->coding = ZVT_CODE_ISOLATIN1;
+#endif
+
   vt->user_data = 0;
   return vt;
 }
@@ -1845,47 +1916,40 @@ vt_resize_lines(struct vt_line *wn, int width, uint32 default_attr)
     /* d(printf("wn->line=%d wn->width=%d width=%d\n", wn->line, wn->width, width)); */
     
     /* terminal width grew */
-    if (wn->width < width)
-      {
-	/* get the attribute of the last charactor for fill */
-	if (wn->width > 0)
-	  {
-	    c = wn->data[wn->width-1] & VTATTR_MASK;
-	  }
-	else
-	  {
-	    c = default_attr;
-	  }
-
-	/* resize the line */
-	wn = g_realloc(wn, VT_LINE_SIZE(width));
-	
-	/* re-link line into linked list */
-	wn->next->prev = wn;
-	wn->prev->next = wn;
-	
-	/* if the line got bigger, fix it up */
-	for (i = wn->width; i < width; i++) 
-	  {
-	    wn->data[i] = c;
-	    wn->modcount++;
-	    }
-	
-	wn->width = width;
+    if (wn->width < width) {
+      /* get the attribute of the last charactor for fill */
+      if (wn->width > 0)
+	c = wn->data[wn->width-1] & VTATTR_MASK;
+      else
+	c = default_attr;
+      
+      /* resize the line */
+      wn = g_realloc(wn, VT_LINE_SIZE(width));
+      
+      /* re-link line into linked list */
+      wn->next->prev = wn;
+      wn->prev->next = wn;
+      
+      /* if the line got bigger, fix it up */
+      for (i = wn->width; i < width; i++) {
+	wn->data[i] = c;
+	wn->modcount++;
       }
+	
+      wn->width = width;
+    }
     
     /* terminal shrunk */
-    if (wn->width > width) 
-      {
-	/* resize the line */
-	wn = g_realloc(wn, VT_LINE_SIZE(width));
-	
-	/* re-link line into linked list */
-	wn->next->prev = wn;
-	wn->prev->next = wn;
-	
-	wn->width = width;
-      }
+    if (wn->width > width) {
+      /* resize the line */
+      wn = g_realloc(wn, VT_LINE_SIZE(width));
+      
+      /* re-link line into linked list */
+      wn->next->prev = wn;
+      wn->prev->next = wn;
+      
+      wn->width = width;
+    }
     
     wn = nn;
     nn = nn->next;
@@ -1934,46 +1998,40 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
        * switch to a mode where we just dispose of the lines at the bottom
        * of the terminal
        */
-      if (vt->cursory==0)
-	{
-	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines)) ) {
-	    g_free(wn);
-	  }
+      if (vt->cursory==0) {
+	if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines)) )
+	  g_free(wn);
+	
+	/* and for 'alternate' screen */
+	if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines_alt)) )
+	  g_free(wn);
 	  
-	  /* and for 'alternate' screen */
-	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines_alt)) ) {
-	    g_free(wn);
-	  }
-	  
-	  /* repeat for backbuffer */
-	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines_back)) )
-	    g_free(wn);
+	/* repeat for backbuffer */
+	if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines_back)) )
+	  g_free(wn);
+      } else {
+	if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines)) ) {
+	  if ((vt->mode & VTMODE_ALTSCREEN)==0)
+	    vt_scrollback_add(vt, wn);
+	  g_free(wn);
 	}
-      else
-	{
-	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines)) ) {
-	    if ((vt->mode & VTMODE_ALTSCREEN)==0)
-	      vt_scrollback_add(vt, wn);
-	    g_free(wn);
-	  }
-	  
-	  /* and for 'alternate' screen */
-	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_alt)) ) {
-	    if ((vt->mode & VTMODE_ALTSCREEN)!=0)
-	      vt_scrollback_add(vt, wn);
-	    g_free(wn);
-	  }
-	  
-	  /* repeat for backbuffer */
-	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_back)) )
-	    g_free(wn);
-
-	  vt->cursory--;
+	
+	/* and for 'alternate' screen */
+	if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_alt)) ) {
+	  if ((vt->mode & VTMODE_ALTSCREEN)!=0)
+	    vt_scrollback_add(vt, wn);
+	  g_free(wn);
 	}
-
+	
+	/* repeat for backbuffer */
+	if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_back)) )
+	  g_free(wn);
+	
+	vt->cursory--;
+      }
       count--;
     }
-
+    
     /* reset the line index of now-missing lines to -1 */
     count = vt->height - height;
     if ((vt->mode & VTMODE_ALTSCREEN)==0)
